==> satellite-system-2015.1-sources.jar: com/xebialabs/deployit/plugin/satellite/SendToSatelliteStep.scala <==
package com.xebialabs.deployit.plugin.satellite

import akka.actor.ExtendedActorSystem
import akka.pattern.ask
import akka.util.Timeout
import com.xebialabs.deployit.engine.tasker
import com.xebialabs.deployit.engine.tasker._
import com.xebialabs.deployit.engine.tasker.satellite.SatellitePath
import com.xebialabs.deployit.plugin.api.flow.{ExecutionContext, Step, StepExitCode}
import com.xebialabs.deployit.plugin.satellite.StreamingSupervisor.UploadRequest
import com.xebialabs.satellite.future.AwaitForever
import com.xebialabs.satellite.protocol._
import com.xebialabs.satellite.serialization.{FilesToUpload, TaskSpecificationSerialization}
import com.xebialabs.xlplatform.satellite.Satellite

import scala.beans.BeanProperty
import scala.concurrent.Future
import scala.concurrent.duration._
import scala.util.{Failure, Success, Try}

case class SendToSatelliteStep(uploaderPath: String, satellitePath: SatellitePath, satelliteHostname: String, @BeanProperty description: String) extends Step with AwaitForever {

  override def execute(ctx: ExecutionContext): StepExitCode = {

    val taskPreparationStep = prepareTask(ctx)

    val registration = taskPreparationStep.map(registerTask(_, ctx))

    registration match {
      case Failure(exception) =>
        ctx.logError(exception.getMessage, exception)
        StepExitCode.FAIL

      case Success(value) =>
        value
    }
  }

  def prepareTask(ctx: ExecutionContext): Try[TaskSpecificationExchange] = {
    val task = ctx.getAttribute(TaskExecutionContext.CACHE_KEY).asInstanceOf[TaskSpecification]
    val taskSerializer = TaskSpecificationSerialization(SatelliteActorSystem.actorSystem.asInstanceOf[ExtendedActorSystem])
    taskSerializer.toBinary(task).flatMap {
      case (bytes, files, artifactClass, stepTypes) =>

        val errorInSendingFiles = sendFiles(task.getId, ctx, files).collect {
          case UploadReply.Error(e) => e
          case AlreadyRegistered => "Task is already registered"
        }

        if (errorInSendingFiles.nonEmpty) {
          Failure(new Throwable(s"Error while sending file to satellite. (${errorInSendingFiles.mkString(", ")})"))
        } else {
          Success(TaskSpecificationExchange(stepTypes.map(_.getName), bytes, task.getId, files.map(_.getId), artifactClass.map(_.getName)))
        }
    }
  }

  def registerTask(taskPreparationStep: TaskSpecificationExchange, ctx: ExecutionContext): StepExitCode = {
    implicit val timeout = Timeout(1.day)
    implicit val actorSystem = SatelliteActorSystem.actorSystem
    implicit val executionContext = actorSystem.dispatcher

    val remoteTaskActor = satellitePath.locate(tasker.satellite.Paths.tasks)

    val sendingTask = (remoteTaskActor ? taskPreparationStep).mapTo[TaskReply].map {
      case Registered =>
        StepExitCode.SUCCESS

      case RegistrationFailedForMissingStep(missingSteps) =>
        ctx.logError(s"Steps of types '${missingSteps.mkString(", ")}' are missing on the satellite. Please keep your plugins in sync.")
        StepExitCode.FAIL

      case RegistrationFailedForUnexpectedError(e) =>
        ctx.logError("Satellite can't deserialize task specification. Please keep your plugins in sync.", e)
        StepExitCode.FAIL

      case RegistrationFailedForMissingFiles(missingFiles) =>
        ctx.logError(s"Files '${missingFiles.mkString(", ")}' are missing on the satellite. Please retry.")
        StepExitCode.FAIL

      case RegistrationFailedForMissingArtifactsImplem(missingArtifactTypes) =>
        ctx.logError(s"Artifact of types '${missingArtifactTypes.mkString(", ")}' are missing on the satellite. Please keep your plugins in sync.")
        StepExitCode.FAIL

      case AlreadyRegistered =>
        ctx.logError("Task is already registered")
        StepExitCode.FAIL
    }

    blockOrThrow(sendingTask)
  }

  def sendFiles(taskId: TaskId, ctx: ExecutionContext, files: FilesToUpload) = {
    implicit val timeout = Timeout(1.day)
    implicit val actorSystem = SatelliteActorSystem.actorSystem
    implicit val executionContext = actorSystem.dispatcher

    val uploader = SatelliteActorSystem.actorSystem.actorSelection(uploaderPath)

    val sendingFiles: Set[Future[Any]] = files.map { artifact =>
      uploader ? UploadRequest(
        taskId,
        artifact.getId,
        artifact.getFile,
        ctx,
        satellitePath,
        satelliteHostname)
    }

    blockOrThrow(Future.sequence(sendingFiles))
  }

  @BeanProperty val order = Step.DEFAULT_ORDER
}

object SendToSatelliteStep {

  def apply(satellite: Satellite): SendToSatelliteStep = new SendToSatelliteStep(
    SatelliteActorSystem.uploader.path.toSerializationFormat,
    SatellitePath(satellite), satellite.getAddress,
    "Sending task to satellite")
}

==> satellite-system-2015.2.0-sources.jar: com/xebialabs/deployit/plugin/satellite/SendToSatelliteStep.scala <==
package com.xebialabs.deployit.plugin.satellite

import java.io.ObjectInputStream

import akka.actor.{ActorSystem, ExtendedActorSystem}
import akka.pattern.ask
import akka.util.Timeout
import com.xebialabs.deployit.engine.tasker
import com.xebialabs.deployit.engine.tasker._
import com.xebialabs.deployit.engine.tasker.satellite.ActorLocator
import com.xebialabs.deployit.plugin.api.flow.{ExecutionContext, Step, StepExitCode}
import com.xebialabs.deployit.plugin.satellite.StreamingSupervisor.UploadRequest
import com.xebialabs.satellite.future.AwaitForever
import com.xebialabs.satellite.protocol._
import com.xebialabs.satellite.serialization.{FilesToUpload, TaskSpecificationSerialization}
import com.xebialabs.xlplatform.satellite.Satellite

import scala.beans.BeanProperty
import scala.concurrent.Future
import scala.concurrent.duration._
import scala.util.{Failure, Success, Try}

case class SendToSatelliteStep(uploaderPath: String, actorLocator: ActorLocator, satelliteHostname: String, @BeanProperty description: String)
                              (implicit @transient var satelliteCommunicatorSystem: ActorSystem)  extends Step with AwaitForever {

  override def execute(ctx: ExecutionContext): StepExitCode = {

    val taskPreparationStep = prepareTask(ctx)

    val registration = taskPreparationStep.map(registerTask(_, ctx))

    registration match {
      case Failure(exception) =>
        ctx.logError(exception.getMessage, exception)
        StepExitCode.FAIL

      case Success(value) =>
        value
    }
  }

  def prepareTask(ctx: ExecutionContext): Try[TaskSpecificationExchange] = {
    val task = ctx.getAttribute(TaskExecutionContext.CACHE_KEY).asInstanceOf[TaskSpecification]
    val taskSerializer = TaskSpecificationSerialization(satelliteCommunicatorSystem.asInstanceOf[ExtendedActorSystem])
    taskSerializer.toBinary(task).flatMap {
      case (bytes, files, artifactClass, stepTypes) =>

        val errorInSendingFiles = sendFiles(task.getId, ctx, files).collect {
          case UploadReply.Error(e) => e
          case AlreadyRegistered => "Task is already registered"
        }

        if (errorInSendingFiles.nonEmpty) {
          Failure(new Throwable(s"Error while sending file to satellite. (${errorInSendingFiles.mkString(", ")})"))
        } else {
          Success(TaskSpecificationExchange(stepTypes.map(_.getName), bytes, task.getId, files.map(_.getId), artifactClass.map(_.getName)))
        }
    }
  }

  def registerTask(taskPreparationStep: TaskSpecificationExchange, ctx: ExecutionContext): StepExitCode = {
    implicit val timeout = Timeout(1.day)
    implicit val executionContext = satelliteCommunicatorSystem.dispatcher

    val remoteTaskActor = actorLocator.locate(tasker.satellite.Paths.tasks)

    val sendingTask = (remoteTaskActor ? taskPreparationStep).mapTo[TaskReply].map {
      case Registered =>
        StepExitCode.SUCCESS

      case RegistrationFailedForMissingStep(missingSteps) =>
        ctx.logError(s"Steps of types '${missingSteps.mkString(", ")}' are missing on the satellite. Please keep your plugins in sync.")
        StepExitCode.FAIL

      case RegistrationFailedForUnexpectedError(e) =>
        ctx.logError("Satellite can't deserialize task specification. Please keep your plugins in sync.", e)
        StepExitCode.FAIL

      case RegistrationFailedForMissingFiles(missingFiles) =>
        ctx.logError(s"Files '${missingFiles.mkString(", ")}' are missing on the satellite. Please retry.")
        StepExitCode.FAIL

      case RegistrationFailedForMissingArtifactsImplem(missingArtifactTypes) =>
        ctx.logError(s"Artifact of types '${missingArtifactTypes.mkString(", ")}' are missing on the satellite. Please keep your plugins in sync.")
        StepExitCode.FAIL

      case AlreadyRegistered =>
        ctx.logError("Task is already registered")
        StepExitCode.FAIL
    }

    blockOrThrow(sendingTask)
  }

  def sendFiles(taskId: TaskId, ctx: ExecutionContext, files: FilesToUpload) = {
    implicit val timeout = Timeout(1.day)
    implicit val executionContext = satelliteCommunicatorSystem.dispatcher

    val uploader = satelliteCommunicatorSystem.actorSelection(uploaderPath)

    val sendingFiles: Set[Future[Any]] = files.map { artifact =>
      uploader ? UploadRequest(
        taskId,
        artifact.getId,
        artifact.getFile,
        ctx,
        actorLocator,
        satelliteHostname)
    }

    blockOrThrow(Future.sequence(sendingFiles))
  }

  @BeanProperty val order = Step.DEFAULT_ORDER

  private def readObject(in: ObjectInputStream): Unit = {
    in.defaultReadObject()
    satelliteCommunicatorSystem = SatelliteCommunicatorSystem.actorSystem
  }
}

object SendToSatelliteStep {

  def apply(satellite: Satellite)(implicit satelliteCommunicatorSystem: ActorSystem): SendToSatelliteStep = new SendToSatelliteStep(
    SatelliteCommunicatorSystem.uploader.path.toSerializationFormat,
    ActorLocator(satellite), satellite.getAddress,
    "Sending task to satellite")
}
